//----------------------------------------------------------------------
//EASTRISING TECHNOLOGY CO,.LTD.//
// Module    : ER-OLEDM015-2  1.5" 
// Language  : C51 SI_SEG_CODE
// Create    : JAVEN
// Date      : Sep-02-2019, Jan-30-2023
// Drive IC  : SSD1309
// INTERFACE : 4Line SPI
// MCU 		 : Was AT89LE52RC, ported to SiLabs C8051F330 by Ian.M
// Notes 	 : Use SDCC +SiLabs Headers toolchain as code size exceeds
//             Keil demo version 2K limit.  The arrays are now compressed
//			   to fit within 'F330 7.75K user code limit.							
//----------------------------------------------------------------------

#include <si_toolchain.h>
#include <C8051F330_defs.h>            // SFR declarations

/*
* // Was (see ER-OLEDM015-2-SPI-I2C Interfacing.pdf):
* #define RS                     P3_3 // OLED 'DC' is Data/nCommand
* #define CS1                    P3_5 // OLED 'CS' is /CS
* #define RST                    P3_4 // OLED 'RES' is OLED Reset
* #define SCLK                  P1_0  // OLED 'SCL' is SCLK
* #define SDA                   P1_1  // OLED 'SDA' is MOSI
* sbit	KEY_PAUSE=P3^2;		// Button 'PAUSE' to Gnd
* sbit KEY_ADD=P0^1;	// Button 'Contrast+' to Gnd
* sbit KEY_DEC=P0^2;	// Button 'Contrast-' to Gnd
*/

// Recommended for hardware SPI on 'F330
// P0.0  -  SCK  (SPI0), Push-Pull,  Digital
// P0.1  -  MISO (SPI0), Open-Drain, Digital
// P0.2  -  MOSI (SPI0), Push-Pull,  Digital
// P0.3  -  nSS  (SPI0), Push-Pull,  Digital

// Reserved pins on F330DC
// P0.4  -  Reserved: Tx to host
// P0.5  -  Reserved: Rx from host
// P0.6  -  RTS from host
// P0.7  -  CTS to host
// P1.1  -  IDAC (1K to Gnd)
// P1.3  -  Green D2 LED K
// P1.6	 -	Pot (ADC)
// P2.0  -  Reserved: CSD (ISP/debug data)

SI_SBIT(P0_0, SFR_P0, 0);
SI_SBIT(P0_2, SFR_P0, 2);
SI_SBIT(P0_3, SFR_P0, 3);

SI_SBIT(P1_0, SFR_P1, 0);
SI_SBIT(P1_1, SFR_P1, 1);
SI_SBIT(P1_2, SFR_P1, 2);
SI_SBIT(P1_3, SFR_P1, 3); 
SI_SBIT(P1_4, SFR_P1, 4);
SI_SBIT(P1_5, SFR_P1, 5);


// Outputs
#define RS	 P1_4 // OLED 'DC' is Data/Command
#define CS1  P0_3 // OLED 'CS' is /CS
#define RST  P1_5 // OLED 'RES' is OLED Reset
#define SCLK P0_0 // OLED 'SCL' is SCLK
#define SDA  P0_2 // OLED 'SDA' is MOSI
#define nLED P1_3 // onboard green LED
// Inputs
#define KEY_PAUSE P1_0 // Button 'PAUSE' to Gnd
#define KEY_ADD   P1_1 // Button 'Contrast+' to Gnd
#define KEY_DEC   P1_2 // Button 'Contrast-' to Gnd

uint8_t Contrast_level=0xa0;
SI_BIT(log)=0;

#define Start_column	0x00
#define Start_page		0x00
#define	StartLine_set	0x00

void Delay1(volatile uint16_t n);
void Write_number(uint8_t SI_SEG_CODE *n,uint8_t k,uint8_t station_dot);
void display_Contrast_level(uint8_t number);
void adj_Contrast(void);
void Delay(uint16_t n);
void Write_Instruction(unsigned char cmd);
void Write_Byte(unsigned char dat);
void Set_Page_Address(unsigned char add);
void Set_Column_Address(unsigned char add);
void Set_Contrast_Control_Register(unsigned char mod);
void Initial(void);
void Display_Chess(unsigned char value);
void Display_Chinese(unsigned char ft[]);
void Display_Chinese_Column(unsigned char ft[]);
void Display_Picture(unsigned char pic[]);


uint8_t SI_SEG_CODE	    num[]={0x00,0xF8,0xFC,0x04,0x04,0xFC,0xF8,0x00,0x00,0x03,0x07,0x04,0x04,0x07,0x03,0x00,  	   /*--  :  0~9  --*/
0x00,0x00,0x08,0xFC,0xFC,0x00,0x00,0x00,0x00,0x00,0x04,0x07,0x07,0x04,0x00,0x00,  
0x00,0x18,0x1C,0x84,0xC4,0x7C,0x38,0x00,0x00,0x06,0x07,0x05,0x04,0x04,0x04,0x00,
0x00,0x08,0x0C,0x24,0x24,0xFC,0xD8,0x00,0x00,0x02,0x06,0x04,0x04,0x07,0x03,0x00,
0x80,0xE0,0x70,0x18,0xFC,0xFC,0x00,0x00,0x00,0x01,0x01,0x05,0x07,0x07,0x04,0x00,
0x00,0x7C,0x7C,0x24,0x24,0xE4,0xC4,0x00,0x00,0x03,0x07,0x04,0x04,0x07,0x03,0x00,
0x00,0xF0,0xF8,0x6C,0x24,0xEC,0xCC,0x00,0x00,0x03,0x07,0x04,0x04,0x07,0x03,0x00,
0x00,0x0C,0x0C,0xC4,0xFC,0x3C,0x04,0x00,0x00,0x00,0x00,0x07,0x07,0x00,0x00,0x00,
0x00,0x98,0xFC,0x64,0x64,0xFC,0x98,0x00,0x00,0x03,0x07,0x04,0x04,0x07,0x03,0x00,
0x00,0x78,0xFC,0x84,0xC4,0xFC,0xF8,0x00,0x00,0x06,0x06,0x04,0x06,0x03,0x01,0x00};


unsigned char SI_SEG_CODE font[]=
{

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x10,0x61,0x06,0xE0,0x00,0x26,0x22,0x1A,0x02,0xC2,0x0A,0x12,0x32,0x06,0x02,0x00,
0x04,0xFC,0x03,0x20,0x20,0x11,0x11,0x09,0x05,0xFF,0x05,0x09,0x19,0x31,0x10,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x10,0x10,0x10,0xFE,0x10,0x10,0xFE,0x00,0x00,0xFC,0x00,0x00,0x00,0xFE,0x00,0x00,
0x08,0x08,0x04,0x47,0x24,0x18,0x07,0x00,0x00,0x1F,0x00,0x00,0x00,0x7F,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x10,0x10,0x10,0xFF,0x10,0xF0,0x00,0x00,0xFE,0x22,0x22,0x22,0x22,0xFE,0x00,0x00,
0x40,0x30,0x0E,0x01,0x00,0x3F,0x40,0x40,0x47,0x44,0x44,0x44,0x44,0x47,0x70,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x00,0x00,0xFE,0x42,0x42,0x42,0x42,0x42,0x42,0x42,0xFE,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x3F,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x3F,0x00,0x00,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x04,0x04,0xC4,0xB4,0x8C,0x87,0x84,0xF4,0x84,0x84,0x84,0x84,0x04,0x00,0x00,
0x00,0x00,0x20,0x18,0x0E,0x04,0x20,0x40,0xFF,0x00,0x02,0x04,0x18,0x30,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x08,0x08,0x08,0x08,0x08,0x08,0xF9,0x4A,0x4C,0x48,0x48,0xC8,0x08,0x08,0x08,0x00,
0x40,0x40,0x20,0x10,0x0C,0x03,0x00,0x00,0x20,0x40,0x40,0x3F,0x00,0x00,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x10,0x12,0x92,0x72,0xFE,0x51,0x91,0x00,0x22,0xCC,0x00,0x00,0xFF,0x00,0x00,0x00,
0x04,0x02,0x01,0x00,0xFF,0x00,0x04,0x04,0x04,0x02,0x02,0x02,0xFF,0x01,0x01,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x08,0x08,0x88,0xFF,0x48,0x28,0x00,0xC8,0x48,0x48,0x7F,0x48,0xC8,0x48,0x08,0x00,
0x01,0x41,0x80,0x7F,0x00,0x40,0x40,0x20,0x13,0x0C,0x0C,0x12,0x21,0x60,0x20,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x00,0xF8,0x48,0x48,0x48,0xC8,0x08,0xFF,0x08,0x09,0x0A,0xC8,0x88,0x08,0x00,
0x40,0x30,0x0F,0x00,0x08,0x50,0x4F,0x20,0x10,0x0B,0x0C,0x12,0x21,0x40,0xF0,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x10,0x10,0x90,0x10,0x10,0x11,0x16,0x10,0x10,0xF0,0x50,0x10,0x10,0x00,0x00,
0x40,0x40,0x40,0x40,0x43,0x4E,0x40,0x60,0x58,0x47,0x40,0x40,0x40,0x60,0x40,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x40,0x40,0x42,0x42,0x42,0x42,0x42,0xFE,0x42,0x42,0x42,0x42,0x42,0x42,0x40,0x00,
0x00,0x00,0x00,0x00,0x20,0x40,0x80,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x00,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x06,0x04,0x00,0x00,0x00,
0x00,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x18,0x10,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x0C,0x04,0x55,0x55,0x55,0x85,0x7F,0x85,0x55,0x55,0x55,0x05,0x0C,0x04,0x00,
0x04,0x04,0x04,0x0A,0x0A,0x09,0x2A,0xCC,0x28,0x19,0x09,0x02,0x06,0x02,0x02,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x0C,0x04,0x55,0x55,0x55,0x85,0x7F,0x85,0x55,0x55,0x55,0x05,0x0C,0x04,0x00,
0x04,0x04,0x04,0x0A,0x0A,0x09,0x2A,0xCC,0x28,0x19,0x09,0x02,0x06,0x02,0x02,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x00,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x06,0x04,0x00,0x00,0x00,
0x00,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x18,0x10,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x40,0x20,0x10,0x0C,0xE3,0x22,0x22,0x22,0xFE,0x22,0x22,0x22,0x22,0x02,0x00,0x00,
0x04,0x04,0x04,0x04,0x07,0x04,0x04,0x04,0xFF,0x04,0x04,0x04,0x04,0x04,0x04,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x04,0xFF,0x54,0x54,0x54,0xFF,0x04,0x00,0xFE,0x22,0x22,0x22,0xFE,0x00,0x00,
0x42,0x22,0x1B,0x02,0x02,0x0A,0x33,0x62,0x18,0x07,0x02,0x22,0x42,0x3F,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x20,0x10,0x08,0xC7,0x3A,0x50,0x44,0x44,0x44,0x7F,0x44,0xC4,0x44,0x44,0x40,0x00,
0x00,0x02,0x01,0xFF,0x00,0x02,0x02,0x1A,0x02,0x42,0x82,0x7F,0x02,0x02,0x02,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x44,0x54,0x54,0x55,0x56,0xD4,0x7C,0x54,0x54,0x56,0x55,0x54,0x44,0x44,0x00,
0x08,0x08,0x04,0x04,0x02,0xFF,0x55,0x55,0x55,0x55,0x55,0x55,0xFF,0x00,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x80,0x40,0x30,0xFC,0x03,0x90,0x68,0x06,0x04,0xF4,0x04,0x24,0x44,0x8C,0x04,0x00,
0x00,0x20,0x38,0x03,0x38,0x40,0x40,0x49,0x52,0x41,0x40,0x70,0x00,0x09,0x30,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x80,0x84,0x84,0x94,0xA4,0xC4,0xFF,0x84,0xC4,0xB4,0x94,0x84,0x84,0x80,0x00,
0x20,0x20,0x10,0x10,0x08,0x04,0x02,0xFF,0x01,0x02,0x04,0x08,0x18,0x30,0x10,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x00,0xF8,0x48,0x48,0x48,0x48,0xFF,0x48,0x48,0x48,0x48,0xF8,0x00,0x00,0x00,
0x00,0x00,0x0F,0x04,0x04,0x04,0x04,0x3F,0x44,0x44,0x44,0x44,0x4F,0x40,0x70,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x80,0x84,0x84,0x94,0xA4,0xC4,0xFF,0x84,0xC4,0xB4,0x94,0x84,0x84,0x80,0x00,
0x20,0x20,0x10,0x10,0x08,0x04,0x02,0xFF,0x01,0x02,0x04,0x08,0x18,0x30,0x10,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0xE0,0x02,0x12,0xE2,0x42,0x02,0xF2,0x12,0x8A,0x66,0x47,0x02,0xE0,0x00,0x00,
0x00,0x7F,0x20,0x24,0x24,0x2A,0x31,0x2F,0x21,0x22,0x24,0x2C,0x20,0x7F,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x20,0x20,0x22,0x22,0xFE,0x21,0x21,0x20,0x20,0xFF,0x20,0x22,0xAC,0x20,0x20,0x00,
0x04,0x04,0x42,0x82,0x7F,0x01,0x01,0x10,0x10,0x08,0x07,0x1A,0x21,0x40,0xF0,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x80,0x40,0xF0,0x1C,0x03,0xF0,0x01,0x0E,0x04,0x00,0x04,0x04,0x04,0xFC,0x00,0x00,
0x00,0x00,0x7F,0x00,0x00,0x7F,0x00,0x00,0x00,0x00,0x00,0x20,0x40,0x3F,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x04,0x04,0x04,0xF4,0x54,0x5C,0x57,0x54,0x54,0x54,0xF4,0x04,0x06,0x04,0x00,
0x10,0x90,0x90,0x50,0x5F,0x35,0x15,0x15,0x15,0x35,0x55,0x5F,0x90,0x90,0x10,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x20,0x21,0xE6,0x02,0x08,0xF8,0x88,0x88,0x88,0x08,0xFF,0x08,0x0A,0xCC,0x88,0x00,
0x00,0x00,0x3F,0x90,0x68,0x1F,0x20,0x40,0x3F,0x20,0x10,0x0F,0x14,0x23,0x78,0x00,

/*--  :  Ϊ  --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x10,0x10,0x12,0x14,0x1C,0x10,0xF0,0x9F,0x10,0x10,0x10,0x10,0xF8,0x10,0x00,
0x00,0x00,0x40,0x20,0x10,0x08,0x06,0x01,0x00,0x11,0x26,0x40,0x20,0x1F,0x00,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x80,0x40,0x30,0xFC,0x03,0x90,0x68,0x06,0x04,0xF4,0x04,0x24,0x44,0x8C,0x04,0x00,
0x00,0x20,0x38,0x03,0x38,0x40,0x40,0x49,0x52,0x41,0x40,0x70,0x00,0x09,0x30,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x00,0x00,0xFF,0x91,0x91,0xFF,0x00,0x00,0xFF,0xC1,0x41,0x49,0x51,0xCF,0x00,0x00,
0x40,0x30,0x0F,0x20,0x40,0x3F,0x00,0x00,0x7F,0x21,0x16,0x08,0x36,0x61,0x20,0x00,

/*--  :    --*/
/*--  12;  ¶ӦĵΪx=16x16   --*/
0x20,0x10,0x10,0x88,0x84,0x4F,0x52,0xA2,0x22,0x52,0x4A,0x86,0x80,0x80,0x80,0x00,
0x80,0x81,0x41,0x42,0x22,0x12,0x0E,0x03,0x02,0x42,0x82,0x7E,0x00,0x01,0x00,0x00

};

// Insert five compressed picture arrays from seperate file
#include "RLCarrays.h"

//void Step(void) interrupt 0 using 0
SI_INTERRUPT(Step, INTERRUPT_INT1)
{
	Delay1(300);
	if(KEY_PAUSE==0)
 // {	while(IE0);
	log=!log;
 // }
   while(!KEY_PAUSE);
  return;

}



void Delay1(volatile uint16_t n)
{while(n--); 
	return;
}


void Write_number(uint8_t SI_SEG_CODE *n,uint8_t k,uint8_t station_dot)
{uint8_t i; 
			RS=1;
			for(i=0;i<8;i++)
				{
				Write_Byte(*(n+16*k+i));
				}
				
	Set_Page_Address(Start_page+1)	;
    Set_Column_Address(Start_column+station_dot*8);
			RS=1;	
			for(i=8;i<16;i++)
				{
				Write_Byte(*(n+16*k+i));
				}
}



void display_Contrast_level(uint8_t number)
{	uint8_t number1,number2,number3;
	number1=number/100;number2=number%100/10;number3=number%100%10;
	Set_Column_Address(Start_column+0*8);
	Set_Page_Address(Start_page);
    Write_number(num,number1,0);
	Set_Column_Address(Start_column+1*8);
	Set_Page_Address(Start_page);
	Write_number(num,number2,1);
	Set_Column_Address(Start_column+2*8);
	Set_Page_Address(Start_page);
	Write_number(num,number3,2);

}


void adj_Contrast(void)
{	Delay1(300);
 	 if((KEY_ADD==0)||(KEY_DEC==0))
		{		
		 if	(KEY_ADD==0) 
			{Contrast_level+=1;
		   	Set_Contrast_Control_Register(Contrast_level);	
			display_Contrast_level(Contrast_level);
			Delay1(18000);
			if(KEY_ADD==0)
				{Delay1(18000);
					if(KEY_ADD==0)			
						{Delay1(18000);
							if(KEY_ADD==0)
								{Delay1(18000);
									if(KEY_ADD==0)

										{while(KEY_ADD==0)
											{Contrast_level+=1; 
		   										Set_Contrast_Control_Register(Contrast_level);	
												display_Contrast_level(Contrast_level);
							 					Delay1(20000); 
											}
										}
								}				
						}						
				}												
			}
	


		  
	
		 if	(KEY_DEC==0) 
			{Contrast_level-=1; 
		   	Set_Contrast_Control_Register(Contrast_level);	
			display_Contrast_level(Contrast_level);
			Delay1(18000);
			if(KEY_DEC==0)
				{Delay1(18000);
					if(KEY_DEC==0)			
						{Delay1(18000);
							if(KEY_DEC==0)
								{Delay1(18000);
									if(KEY_DEC==0)

										{while(KEY_DEC==0)
											{Contrast_level-=1; 
		   										Set_Contrast_Control_Register(Contrast_level);	
												display_Contrast_level(Contrast_level);
							 					Delay1(20000); 
											}
										}
								}				
						}						
				}												
			}

	  	}
}


void Delay(uint16_t n){

	nLED=0;
	while(n){
    	if((KEY_ADD==0)||(KEY_DEC==0)) adj_Contrast();
		n--;
		for(volatile char m=0;m<60;m++){}
 	}
 	for(;log==1;){
 		if((KEY_ADD==0)||(KEY_DEC==0)) adj_Contrast();
	}
	nLED=1;
	return;
}


/*
void Write_Data(unsigned char dat) // original *SLOOOOOW* code
{
  unsigned char  m,da;
  unsigned int j;
     RS=1;
     CS1=0;
     da=dat;
  for(j=0;j<8;j++)
    {
      m=da;
      SCLK=0;
      m=m&0x80;
      if(m==0x80)
        {
           SDA=1;
         }
       else
         {
             SDA=0;
         }
        
        da=da<<1;
        SCLK=1;
      }
  CS1=1;
}
*/


/*
void Write_Byte(unsigned char dat) // faster but still bit-banged
{
  unsigned char  m;
     CS1=0;
  for(m=0x80;m;m=m>>1)
    {
      SCLK=0;
      if(dat&m)
        {
           SDA=1;
         }
       else
         {
             SDA=0;
         }
        SCLK=1;
      }
  CS1=1;
}
*/

void Write_Byte(unsigned char dat){ // HW SPI
	CS1=0;			// assert /SS
	SPI0DAT=dat;	// Send data
	while (!SPIF);	// wait for end of transfer
	SPIF=0;			// clear SPI interrupt flag
	CS1=1;			// deassert /SS
}

void Write_Instruction(unsigned char cmd){

	RS=0;
	Write_Byte(cmd);
}


// Set page address 
void Set_Page_Address(unsigned char add)
{
    add=0xb0|add;
    Write_Instruction(add);
	return;
}

void Set_Column_Address(unsigned char add)
{
	Write_Instruction((0x10|(add>>4)));
	Write_Byte((0x0f&add));
	return;
}



void Set_Contrast_Control_Register(unsigned char mod)
{
	Write_Instruction(0x81);
	Write_Byte(mod);
	return;
}


//#pragma disable  // no interrupts
void Initial(void)
{
	RST=1;
	Delay(2000);
	RST=0;
	Delay(2000);
	RST=1;

	Delay(2000);

 	Write_Instruction(0xae);//--turn off oled panel

	Write_Byte(0xd5);//--set display clock divide ratio/oscillator frequency
	Write_Byte(0x80);//--set divide ratio

	Write_Byte(0xa8);//--set multiplex ratio(1 to 64)
	Write_Byte(0x3f);//--1/64 duty

	Write_Byte(0xd3);//-set display offset
	Write_Byte(0x00);//-not offset


	Write_Byte(0x40);//--set start line address

	Write_Byte(0xa6);//--set normal display

	Write_Byte(0xa4);//Disable Entire Display On

	Write_Byte(0xa1);//--set segment re-map 128 to 0

	Write_Byte(0xC8);//--Set COM Output Scan Direction 64 to 0

	Write_Byte(0xda);//--set com pins hardware configuration
	Write_Byte(0x12);

	Write_Byte(0x81);//--set contrast control register
	Write_Byte(Contrast_level);


	Write_Byte(0xd9);//--set pre-charge period
	Write_Byte(0x25);

	Write_Byte(0xdb);//--set vcomh
	Write_Byte(0x34);



	Write_Byte(0xaf);//--turn on oled panel
}



void Display_Chess(unsigned char value)
{
    unsigned char i,j,k;
    for(i=0;i<0x08;i++)
	{
		Set_Page_Address(i);

        Set_Column_Address(0x00);

		RS=1;
		for(j=0;j<0x10;j++)
		{
		    for(k=0;k<0x04;k++)
		        Write_Byte(value);
		    for(k=0;k<0x04;k++)
		        Write_Byte(~value);
		}
	}
    return;
}


void Display_Chinese(unsigned char ft[])
{
    unsigned char i,j,k,b,c=0;
	unsigned int	num=0;

for(b=0;b<4;b++)
  {
    for(i=0;i<0x02;i++)
	{	Set_Page_Address(c);
    	Set_Column_Address(0x00);
	    num=i*0x10+b*256;
		RS=1;
		for(j=0;j<0x08;j++)
		{
            for(k=0;k<0x10;k++)
			{
		        Write_Byte(ft[num+k]);
			}
			num+=0x20;
		}c++;
	}
  }
    return;
}


//Display_Chinese1
void Display_Chinese_Column(unsigned char ft[])
{
    unsigned char i,j,k,num=0x40;
    for(i=0;i<0x08;i++)
	{
		Set_Page_Address(i);
        Set_Column_Address(0x00);
		RS=1;
		for(j=0;j<0x08;j++)
		{
            for(k=0;k<0x10;k++)
			{
		        Write_Byte(ft[num+k]);
			}
		}
	num+=0x10;
	}
    return;
}


void Display_Picture(unsigned char cpic[]){
    unsigned char i,j,num=0,esc,curr,prev=0,count=0;
	int idx=1;

	esc=cpic[0];
	for(i=0;i<0x08;i++){

		Set_Page_Address(i);
    	Set_Column_Address(0x00);
    	RS=1;

		for(j=0;j<0x80;j++){
			if(!count){ // get a new byte
				curr=cpic[idx++];
				if(curr==esc){ 
					count=cpic[idx++];
					if(!count){ // expand escaped esc
						Write_Byte(esc);
						prev=esc;
					} //fi(!count)
				}else{
					Write_Byte(curr);
					prev=curr;
				} //fi(curr==esc)
			} //fi(!count) outer 
			if(count){ // repeat in a run
				Write_Byte(prev);
				count--;
			} //fi(count)
		} //rof(j)

	} //rof(i)
    return;
}



void main(void)
{	
	// 'F330 Hardware Setup 'Magic' numbers generated by Config2 utility
	// See: ER_OLEDM015_2_HWSPI.cwg - SFR lines extracted and pasted here.
    OSCICN    = 0x83;  // 24.5 MHz IntOsc

    PCA0MD    &= ~0x40; // init PCA (disable WDT)
    PCA0MD    = 0x00;

    SPI0CFG   = 0x70; // init HW SPI - 3 wire master, mode 3
    SPI0CN    = 0x01; // bitbang /SS as port pin
	SPI0CKR   = 0x01; // 1/4 SYSCLK - 6.125MHz  (<10MHz SSD1309 max.)


    P0MDOUT   = 0x0D; // init ports
    P1MDOUT   = 0x38;
	XBR0      = 0x02; // Init crossbar
    XBR1      = 0x40;

    IT01CF    = 0x00; // configure INT1
    IE        = 0x84;

	KEY_PAUSE=1;
	Delay(100);

    Initial();

	while(1)
	{

        Display_Picture(RLCpic);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Picture(RLCpic1);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Picture(RLCpic2);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Picture(RLCpic3);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Picture(RLCpic4);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Chess(0x0f);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Chinese(font);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Display_Chinese(font);
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
		Display_Chinese_Column(font);
		Delay(65000);

		Write_Instruction(0xa7);//--set Inverse Display	
		Display_Chinese_Column(font);
		Delay(65000);

		Write_Instruction(0xa6);//--set normal display
	}
}
